@@ -151,7 +151,7 @@ where
     I: InputType + Neg<Output = I>,
     F: OutputType + Neg<Output = F>,
 {
-    #[allow(dead_code)]
+    #[cfg(test)]
     /// only used by unit test code
     pub(crate) fn new_2(point: Point<I>, initial_index: SiteEventIndexType) -> SiteEvent<I, F> {
         Self {
@@ -182,7 +182,7 @@ where
     }
 
     /// Only used by test code
-    #[allow(dead_code)]
+    #[cfg(test)]
     pub fn new_7(
         x1: I,
         y1: I,
@@ -202,6 +202,26 @@ where
         }
     }
 
+    #[cfg(feature = "ce_corruption_check")]
+    #[allow(dead_code)]
+    pub fn dbg(&self) {
+        if self.is_point() {
+            println!(
+                "[{},{}];",
+                num::cast::<I, f64>(self.point0().x).unwrap(),
+                num::cast::<I, f64>(self.point0().y).unwrap()
+            );
+        } else {
+            println!(
+                "[{},{},{},{}];",
+                num::cast::<I, f64>(self.point0().x).unwrap(),
+                num::cast::<I, f64>(self.point0().y).unwrap(),
+                num::cast::<I, f64>(self.point1().x).unwrap(),
+                num::cast::<I, f64>(self.point1().y).unwrap()
+            );
+        }
+    }
+
     #[inline(always)]
     pub fn x(&self) -> I {
         self.point0_.x
@@ -279,7 +299,7 @@ where
 
     /// only for basic test purposes
     #[inline(always)]
-    #[allow(dead_code)]
+    #[cfg(test)]
     pub(crate) fn set_flags(&mut self, flags: u32) {
         self.flags_ = flags;
     }
@@ -314,6 +334,22 @@ where
         }
         !(site1.is_segment() ^ site2.is_segment())
     }
+
+    #[cfg(all(feature = "ce_corruption_check", feature = "geo"))]
+    #[inline(always)]
+    pub fn distance_to_point(&self, x: f64, y: f64) -> f64 {
+        use geo::algorithm::euclidean_distance::*;
+        let c = geo::Coordinate { x, y };
+
+        if self.is_point() {
+            c.euclidean_distance(&geo::Coordinate::from(self.point0().as_f64()))
+        } else {
+            c.euclidean_distance(&geo::Line::new(
+                geo::Coordinate::from(self.point0().as_f64()),
+                geo::Coordinate::from(self.point1().as_f64()),
+            ))
+        }
+    }
 }
 
 impl<I, O> fmt::Display for SiteEvent<I, O>
