@@ -879,124 +879,6 @@ fn two_segments_3() -> Result<(), BvError> {
     Ok(())
 }
 
-//#[ignore]
-#[test]
-/// three segments and one point
-fn two_segments_4() -> Result<(), BvError> {
-    let output = {
-        let _v = vec![Point { x: 4, y: 3 }];
-        let _s = vec![
-            Line::new(Point { x: 1, y: 2 }, Point { x: 3, y: 4 }),
-            Line::new(Point { x: 2, y: 2 }, Point { x: 5, y: 4 }),
-            Line::new(Point { x: 5, y: 6 }, Point { x: 3, y: 1 }),
-        ];
-        let mut vb = VB::Builder::<I, F>::default();
-        vb.with_vertices(_v.iter())?;
-        vb.with_segments(_s.iter())?;
-        vb.build()?
-    };
-    assert_eq!(output.cells().len(), 10);
-    let cell = output.cells()[0].get();
-    assert_eq!(cell.id().0, 0);
-    assert_eq!(cell.source_index(), 1);
-    assert_eq!(cell.is_degenerate(), false);
-    assert_eq!(cell.contains_point(), true);
-    assert_eq!(cell.contains_segment(), false);
-    let cell = output.cells()[1].get();
-    assert_eq!(cell.id().0, 1);
-    assert_eq!(cell.source_index(), 1);
-    assert_eq!(cell.is_degenerate(), false);
-    assert_eq!(cell.contains_point(), false);
-    assert_eq!(cell.contains_segment(), true);
-    let cell = output.cells()[2].get();
-    assert_eq!(cell.id().0, 2);
-    assert_eq!(cell.source_index(), 2);
-    assert_eq!(cell.is_degenerate(), false);
-    assert_eq!(cell.contains_point(), true);
-    assert_eq!(cell.contains_segment(), false);
-    let cell = output.cells()[3].get();
-    assert_eq!(cell.id().0, 3);
-    assert_eq!(cell.source_index(), 2);
-    assert_eq!(cell.is_degenerate(), false);
-    assert_eq!(cell.contains_point(), false);
-    assert_eq!(cell.contains_segment(), true);
-    let cell = output.cells()[4].get();
-    assert_eq!(cell.id().0, 4);
-    assert_eq!(cell.source_index(), 3);
-    assert_eq!(cell.is_degenerate(), false);
-    assert_eq!(cell.contains_point(), true);
-    assert_eq!(cell.contains_segment(), false);
-    let cell = output.cells()[5].get();
-    assert_eq!(cell.id().0, 5);
-    assert_eq!(cell.source_index(), 1);
-    assert_eq!(cell.is_degenerate(), false);
-    assert_eq!(cell.contains_point(), true);
-    assert_eq!(cell.contains_segment(), false);
-    let cell = output.cells()[6].get();
-    assert_eq!(cell.id().0, 6);
-    assert_eq!(cell.source_index(), 3);
-    assert_eq!(cell.is_degenerate(), false);
-    assert_eq!(cell.contains_point(), false);
-    assert_eq!(cell.contains_segment(), true);
-    let cell = output.cells()[7].get();
-    assert_eq!(cell.id().0, 7);
-    assert_eq!(cell.source_index(), 0);
-    assert_eq!(cell.is_degenerate(), false);
-    assert_eq!(cell.contains_point(), true);
-    assert_eq!(cell.contains_segment(), false);
-    let cell = output.cells()[8].get();
-    assert_eq!(cell.id().0, 8);
-    assert_eq!(cell.source_index(), 2);
-    assert_eq!(cell.is_degenerate(), false);
-    assert_eq!(cell.contains_point(), true);
-    assert_eq!(cell.contains_segment(), false);
-    let cell = output.cells()[9].get();
-    assert_eq!(cell.id().0, 9);
-    assert_eq!(cell.source_index(), 3);
-    assert_eq!(cell.is_degenerate(), false);
-    assert_eq!(cell.contains_point(), true);
-    assert_eq!(cell.contains_segment(), false);
-    assert_eq!(output.vertices().len(), 12);
-    assert_eq!(output.edges().len(), 42);
-    let v = output.vertices()[0].get();
-    assert!(almost_equal(v.x(), 1.80196, v.y(), 2.29706));
-    assert_eq!(v.get_incident_edge()?.0, 7);
-    let v = output.vertices()[1].get();
-    assert!(almost_equal(v.x(), 1.50000, v.y(), 1.50000));
-    assert_eq!(v.get_incident_edge()?.0, 9);
-    let v = output.vertices()[2].get();
-    assert!(almost_equal(v.x(), 2.28571, v.y(), 1.28571));
-    assert_eq!(v.get_incident_edge()?.0, 17);
-    let v = output.vertices()[3].get();
-    assert!(almost_equal(v.x(), 1.50000, v.y(), 0.500000));
-    assert_eq!(v.get_incident_edge()?.0, 19);
-    let v = output.vertices()[4].get();
-    assert!(almost_equal(v.x(), 2.39530, v.y(), 1.40706));
-    assert_eq!(v.get_incident_edge()?.0, 21);
-    let v = output.vertices()[5].get();
-    assert!(almost_equal(v.x(), 3.39608, v.y(), 3.60392));
-    assert_eq!(v.get_incident_edge()?.0, 23);
-    let v = output.vertices()[6].get();
-    assert!(almost_equal(v.x(), 3.97999, v.y(), 5.02001));
-    assert_eq!(v.get_incident_edge()?.0, 31);
-    let v = output.vertices()[7].get();
-    assert!(almost_equal(v.x(), 4.75205, v.y(), 3.24795));
-    assert_eq!(v.get_incident_edge()?.0, 33);
-    let v = output.vertices()[8].get();
-    assert!(almost_equal(v.x(), -3.78095, v.y(), 9.51238));
-    assert_eq!(v.get_incident_edge()?.0, 35);
-    let v = output.vertices()[9].get();
-    assert!(almost_equal(v.x(), 7.50000, v.y(), 5.00000));
-    assert_eq!(v.get_incident_edge()?.0, 37);
-    let v = output.vertices()[10].get();
-    assert!(almost_equal(v.x(), 8.13684, v.y(), -0.136838));
-    assert_eq!(v.get_incident_edge()?.0, 39);
-    let v = output.vertices()[11].get();
-    assert!(almost_equal(v.x(), 11.1250, v.y(), -2.25000));
-    assert_eq!(v.get_incident_edge()?.0, 41);
-    Ok(())
-}
-
 //#[ignore]
 #[test]
 /// two segments and four points
